#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include "nrf.h"
#include "nrf_drv_saadc.h"
#include "boards.h"
#include "app_error.h"
#include "nrf_delay.h"
#include "app_util_platform.h"

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#define SAMPLES_IN_BUFFER 1

static nrf_saadc_value_t    m_buffer;
static uint32_t             m_adc_evt_counter;
static volatile bool        m_sampling = false;


static void saadc_callback(nrf_drv_saadc_evt_t const * p_event)
{
    if (p_event->type == NRF_DRV_SAADC_EVT_DONE)
    {
        ret_code_t err_code;

        err_code = nrf_drv_saadc_buffer_convert(p_event->data.done.p_buffer, SAMPLES_IN_BUFFER);
        APP_ERROR_CHECK(err_code);

        NRF_LOG_INFO("ADC event number: %d", (int)m_adc_evt_counter);
        NRF_LOG_INFO("%d", p_event->data.done.p_buffer[0]);

        m_adc_evt_counter++;

        m_sampling = false;
    }
}


static void saadc_init(void)
{
    ret_code_t err_code;
    nrf_saadc_channel_config_t channel_config =
        NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(NRF_SAADC_INPUT_AIN0);

    err_code = nrf_drv_saadc_init(NULL, saadc_callback);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_drv_saadc_channel_init(0, &channel_config);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_drv_saadc_buffer_convert(&m_buffer, SAMPLES_IN_BUFFER);
    APP_ERROR_CHECK(err_code);
}


static void saadc_uninit(void)
{
    nrf_drv_saadc_uninit();
}


static void saadc_sample(void)
{
    m_sampling = true;

    ret_code_t err_code = nrf_drv_saadc_sample();
    APP_ERROR_CHECK(err_code);
}


static void saadc_init_sample_uninit(void)
{
    saadc_init();
    saadc_sample();
    while (m_sampling == true) {};
    saadc_uninit();
}


int main(void)
{
    ret_code_t err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);

    NRF_LOG_DEFAULT_BACKENDS_INIT();

    NRF_LOG_INFO("SAADC HAL simple example started.");

    while (1)
    {
        saadc_init_sample_uninit();
        NRF_LOG_FLUSH();
        nrf_delay_ms(100);
    }
}
